/*
* Licensed Materials - Property of IBM
* 5648-C63
* (C) Copyright IBM Corp. 1999, 2000
*/

/* Includes */
/* <malloc.h> is not available on /390 */
#ifndef __MVS__
#include <malloc.h>
#endif
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include "PluginUtil.h"
#define MAX_PERL_SIZE 0xFFF0
char *readPerlScript(char *filename) {
   FILE *perlFile;
   char *perlScript=0;
   if (filename) {
      printf("Open file %s\n",filename); 
      perlFile=fopen(filename,"r");
     
      perlScript=(char *)malloc(MAX_PERL_SIZE+1);
      if (perlFile!=0) {
         int length;
         memset(perlScript,0x00,MAX_PERL_SIZE+1);
         length=fread(perlScript,1,MAX_PERL_SIZE,perlFile);
         perlScript[length]=0x00;
         printf("Script length %d\n",strlen(perlScript)); 
         fclose(perlFile);
      }
   }
   return perlScript;
}
/**********************************************************************************************/
/* Plug-in node utility functions                                                             */
/*   CciCharIsSpace : Returns 0 if CciChar is not space, otherwise returns 1                  */
/*   CciCharIsAlnum : Returns 0 if CciChar is not alpha-numberic, otherwise returns 1         */
/*   CciCharCpy     : Copy CciChar strings, like strcpy                                       */
/*   CciCharNcpy    : Copy CciChar strings, like strncpy                                      */
/*   CciCharCmp     : Compare CciChar strings, like strcmp                                    */
/*   CciCharLen     : Find length of CciString, like strlen, result in characters (not bytes) */
/*   CciString      : Convert an ordinary string to Unicode, result must be free()'d          */
/*                    after use, intended for use with source-code constants                  */
/*   CciNString     : Convert N bytes of a string to Unicode, result must be free()'d         */
/*                    after use, intended for use with source-code constants                  */
/*   mbString       : Convert Unicode to an ordinary string, result must be free()'d          */
/*                    after use, intended for use to produce human-readable text              */
/*                                                                                            */
/**********************************************************************************************/
int CciCharIsSpace(
                   const CciChar c
                   ){
#ifdef __MVS__
   if (c == chSpace)
#else
      if (((c >= chTab) && (c <= chCR)) || (c == chSpace))
#endif
         {
         return 1;
      }
      else
         {
         return 0;
      }
}

int CciCharIsAlnum(
                   const CciChar c
                   ){
#ifdef __MVS__
   if (((c >= chDigit_0) && (c <= chDigit_9)) ||
      ((c >= chLatin_A) && (c <= chLatin_I)) ||
      ((c >= chLatin_J) && (c <= chLatin_R)) ||
      ((c >= chLatin_S) && (c <= chLatin_Z)) ||
      ((c >= chLatin_a) && (c <= chLatin_i)) ||
      ((c >= chLatin_j) && (c <= chLatin_r)) ||
      ((c >= chLatin_s) && (c <= chLatin_z)))
#else
      if (((c >= chDigit_0) && (c <= chDigit_9)) ||
         ((c >= chLatin_A) && (c <= chLatin_Z)) ||
         ((c >= chLatin_a) && (c <= chLatin_z)))
#endif
         {
         return 1;
      }
      else
         {
         return 0;
      }
}

void CciCharCpy(
                CciChar*       target,
                const CciChar* source
                ){
   CciChar c ;
   for (;;)
      {
      c = *source ;
      source += 1 ;
      *target = c ;
      target += 1 ;
      if ( 0 == c ) break ;
   }
}

void CciCharNCpy(
                 CciChar*       target,
                 const CciChar* source,
                 unsigned int   size)
   {
   CciChar c ;
   int i;
   for (i=0;i<(int)size;i+=1)
      {
      c = *source ;
      *target = c ;
      target += 1 ;
      if ( 0 != c ) source += 1 ;
   }
}

int CciCharLen(
               const CciChar* s
               ){
   int i=0 ;
   while (*s != 0 )
      {
      s += 1 ;
      i += 1 ;
   }
   return i ;
}

int CciCharCmp(
               const CciChar* s1,
               const CciChar* s2
               ){
   CciChar c1 ;
   CciChar c2 ;
   for (;;)
      {
      c1 = *s1 ;
      s1 += 1 ;
      c2 = *s2 ;
      s2 += 1 ;
      if ( (0 == c1) || (c1 != c2) ) break ;
   }
   return c1-c2 ;
}

const CciChar* CciString(
                         const char* source,
                         int         codepage
                         ){
   /* Maximum number of characters in Unicode representation */
   int maxChars = strlen(source) + 1 ;
   CciChar* buffer = (CciChar*)malloc(maxChars * sizeof(CciChar)) ;
   int rc ;
   cciMbsToUcs(&rc, source, buffer, maxChars, codepage) ;
   return buffer ;
}

const CciChar* CciNString(
                          const char*        source, 
                          const unsigned int size,
                          int                codepage)
   {
   /* Maximum number of characters in Unicode representation */
   /* The bytes may not be NULL terminated so we need to make a copy */
   unsigned int bufferSize = ((size + 1) * sizeof(CciChar));
   int rc;
   CciChar* buffer = (CciChar*)malloc(bufferSize);
   memset(buffer, '\0', bufferSize);
   memcpy(buffer, source, size);
   cciMbsToUcs(&rc, source, buffer, size, codepage) ;
   return buffer ;
}


const char* mbString(
                     const CciChar* source,
                     int            codepage
                     ){
   int charCount = CciCharLen(source) ;
   int maxBytes = (charCount+1) * 9 ;
   char * buffer = (char *)malloc(maxBytes) ;
   int rc ;
   cciUcsToMbs(&rc, source, buffer, maxBytes, codepage) ;
   return buffer ;
}

/*
* ASCIIToUnicode
* UnicodeToASCII
*    These functions convert between single-byte ASCII strings
* and double-byte Unicode strings. We will use the standard "mbs"
* functions if possible, but if these are not available then
* we will cast the characters instead.
*/

#if HAVE_MBR_FUNCTIONS

wchar_t* ASCIIToUnicode(const char* aStr)

   {
   unsigned int len = strlen(aStr);
   wchar_t* outStr = (wchar_t*)malloc(sizeof(wchar_t) * (len + 1));
   mbstowcs(outStr, aStr, len + 1);
   outStr[len] = L'\0';
   return outStr;
}

char* UnicodeToASCII(const wchar_t* lStr)

   {
   unsigned int len = wcslen(lStr);
   char* outStr = (char*)malloc(sizeof(char) * (len + 1));
   wcstombs(outStr, lStr, len + 1);
   outStr[len] = '\0';
   return outStr;
}

#else

wchar_t* ASCIIToUnicode(const char* aStr)

   {
   wchar_t* retVal;
   char* inPtr;
   wchar_t* outPtr;
   unsigned int len;
   
   len = strlen(aStr);
   retVal = (wchar_t*)malloc(sizeof(wchar_t) * (len + 1));
   outPtr = retVal;
   inPtr = (char*)aStr;
   
   while (*inPtr != '\0')
      {
      *outPtr = (wchar_t)*inPtr;
      outPtr++;
      inPtr++;
   }
   *outPtr = L'\0';
   return retVal;
}

char* UnicodeToASCII(const wchar_t* lStr)

   {
   char* retVal;
   char* outPtr;
   wchar_t* inPtr;
   unsigned int len;
   
   len = wcslen(lStr);
   retVal = (char*)malloc(len + 1);
   outPtr = retVal;
   inPtr = (wchar_t*)lStr;
   
   while (*inPtr != L'\0')
      {
      *outPtr = (char)*inPtr;
      outPtr++;
      inPtr++;
   }
   *outPtr = '\0';
   return retVal;
}

#endif /* HAVE_MBR_FUNCTIONS */
