/*File : perlnode.i 
* This software includes contributions that are Copyright (c) 1998-2002 University of Chicago.                          
* All rights reserved.
*/
%module wmqi

%include typemaps.i

%{
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <bipcni.h>
#include "perlnode.h"
%}

// Do not expose the retcode as a parameter to Perl

%typemap(in,numinputs=0) WmqiContext *ctx(WmqiContext *ctx) {
	     if (SWIG_ConvertPtr(get_sv("wmqi::context", TRUE), (void **) &ctx, SWIGTYPE_p_WmqiContext,0) < 0) {
                SWIG_croak("Type error in wmqiContext. Expected _p_WmqiContext");
         }
        $1 = ctx;
}


%typemap(argout) WmqiContext *ctx(WmqiContext *ctx) {
	SV *sv;
	ctx=$1; 
    sv=get_sv("wmqi::retcode", TRUE);
    sv_setiv(sv,ctx->retcode);        
}
 
/* use only one parameter when necessary */
%typemap(in) (char *value,int length) {
		$1 = SvPV($input,$2);
}
 
// Creates a new Perl array and places a CciDate into it
%typemap(out) struct CciDate {
             AV *myav;
             SV **svs;
             svs = (SV **) malloc(3*sizeof(SV *));
             svs[0] = sv_newmortal();
             sv_setiv((SV*)svs[0],$1.year);
             svs[1] = sv_newmortal();
             sv_setiv((SV*)svs[1],$1.month);
             svs[2] = sv_newmortal();
             sv_setiv((SV*)svs[2],$1.day);
             myav =  av_make(3,svs);
             free(svs);
             $result = newRV((SV*)myav);
             sv_2mortal($result);
             argvi++;
}
// Creates a new Perl array and places a CciDate into it
%typemap(out) struct CciTime {
             AV *myav;
             SV **svs;
             svs = (SV **) malloc(3*sizeof(SV *));
             svs[0] = sv_newmortal();
             sv_setiv((SV*)svs[0],$1.hour);
             svs[1] = sv_newmortal();
             sv_setiv((SV*)svs[1],$1.minute);
             svs[2] = sv_newmortal();
             sv_setnv((SV*)svs[2],$1.second);
             myav =  av_make(3,svs);
             free(svs);
             $result = newRV((SV*)myav);
             sv_2mortal($result);
             argvi++;
}
// Creates a new Perl array and places a CciDate into it
%typemap(out) struct CciTimestamp {
             AV *myav;
             SV **svs;
             svs = (SV **) malloc(6*sizeof(SV *));
             svs[0] = sv_newmortal();
             sv_setiv((SV*)svs[0],$1.year);
             svs[1] = sv_newmortal();
             sv_setiv((SV*)svs[1],$1.month);
             svs[2] = sv_newmortal();
             sv_setiv((SV*)svs[2],$1.day);
             svs[3] = sv_newmortal();
             sv_setiv((SV*)svs[3],$1.hour);
             svs[4] = sv_newmortal();
             sv_setiv((SV*)svs[4],$1.minute);
             svs[5] = sv_newmortal();
             sv_setnv((SV*)svs[5],$1.second);
             myav =  av_make(6,svs);
             free(svs);
             $result = newRV((SV*)myav);
             sv_2mortal($result);
             argvi++;
}
/* Element type definitions */
#define UNKNOWN       0x00000000
#define NAME          0x01000000
#define VALUE         0x02000000
#define NAME_VALUE    0x03000000

/* Element type masks */
#define MASK_GENERIC  0xFF000000
#define MASK_SPECIFIC 0x00FFFFFF

/* Value type definitions */

#define BOOL          1
#define INTEGER       2
#define REAL          3
#define DECIMAL       4
#define CHAR          5
#define TIME          6
#define GMTTIME       7
#define DATE          8
#define TIMESTAMP     9
#define GMTTIMESTAMP 10
#define BYTEARRAY    12
#define BITARRAY     13

/* Value state definitions */
#define UNDEFINED 0
#define VALID     1
#define INVALID   2

/* Compare mode definitions */
#define COMPARE_MODE_NULL                0
#define COMPARE_MODE_SPECIFIC_TYPE       1
#define COMPARE_MODE_GENERIC_TYPE        2
#define COMPARE_MODE_FULL_TYPE           3
#define COMPARE_MODE_NAME                4
#define COMPARE_MODE_NAME_SPECIFIC_TYPE  5
#define COMPARE_MODE_NAME_GENERIC_TYPE   6
#define COMPARE_MODE_FULL                7

/******************************************************************************/
/*                                                                            */
/* Options                                                                    */
/*                                                                            */
/******************************************************************************/
#define FINALIZE_NONE          0x00000000
#define FINALIZE_VALIDATE      0x00000001
#define SQL_TRANSACTION_AUTO   1
#define SQL_TRANSACTION_COMMIT 2


/******************************************************************************/
/*                                                                            */
/* Utility function return and reason codes                                   */
/*                                                                            */
/******************************************************************************/

#define FAILURE                  -1

#define SUCCESS                   0
#define INV_FACTORY_NAME          8
#define INV_FACTORY_OBJECT        9
#define INV_NODE_NAME            10
#define INV_NODE_OBJECT          11
#define INV_TERMINAL_NAME        12
#define INV_TERMINAL_OBJECT      13
#define INV_MESSAGE_OBJECT       14
#define INV_ELEMENT_OBJECT       15
#define INV_PARSER_NAME          16
#define INV_PARSER_OBJECT        17
#define INV_VFTP                 18
#define INV_DATA_POINTER         19
#define INV_LENGTH               20
#define NAME_EXISTS              21
#define BUFFER_TOO_SMALL         22
#define NO_BUFFER_EXISTS         23
#define PARSER_NAME_TOO_LONG     24
#define NO_EXCEPTION_EXISTS      25
#define INV_MESSAGE_CONTEXT      26
#define INV_DATA_BUFLEN          27
#define MISSING_IMPL_FUNCTION    28
#define INV_OBJECT_NAME          29
#define INV_LOG_TYPE             30
#define INV_SQL_EXPR_OBJECT      31
#define INV_TRANSACTION_TYPE     32
#define INV_STATEMENT            33
#define INV_CODEPAGE             34
#define INV_CHARACTER            35
#define NO_THREADS_AVAILABLE     36
#define INV_NODE_ENV             37
#define INV_IMPL_FUNCTION        38
#define EXCEPTION                64
#define EXCEPTION_UNKNOWN        65
#define EXCEPTION_FATAL          66
#define EXCEPTION_RECOVERABLE    67
#define EXCEPTION_CONFIGURATION  68
#define EXCEPTION_PARSER         69
#define EXCEPTION_CONVERSION     70
#define EXCEPTION_DATABASE       71
#define EXCEPTION_USER           72
#define TIMEOUT                 100
#define SUCCESS_CONTINUE        101
#define SUCCESS_RETURN          102
#define FAILURE_CONTINUE        103
#define FAILURE_RETURN          104

/*Let's just grab definitions adapted from original header file here */

/*Get the root element for a message */
CciElement  *RootElement(
  WmqiContext *ctx,
  CciMessage   *message
);


/*Create an element before the specified element */
CciElement  *CreateElementBefore(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Create an element before the specified element */
CciElement  *CreateElementBeforeUsingParser(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *parserClassName
);

/*Create an element after the specified element */
CciElement  *CreateElementAfter(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Create an element after the specified element */
CciElement  *CreateElementAfterUsingParser(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *parserClassName
);

/*Create an element as a first child of the specified element */
CciElement  *CreateElementAsFirstChild(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Create an element as a first child of the specified element */
CciElement  *CreateElementAsFirstChildUsingParser(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *parserClassName
);

/*Create an element as a last child of the specified element */
CciElement  *CreateElementAsLastChild(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Create an element as a last child of the specified element */
CciElement  *CreateElementAsLastChildUsingParser(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *parserClassName
);

/*Add an element before a target element */
void   AddBefore(
  WmqiContext *ctx,
  CciElement *targetElement,
  CciElement *newElement
);

/*Add an element after a target element */
void   AddAfter(
  WmqiContext *ctx,
  CciElement *targetElement,
  CciElement *newElement
);

/*Add an element as a first child of a target element */
void   AddAsFirstChild(
  WmqiContext *ctx,
  CciElement *targetElement,
  CciElement *newElement
);

/*Add an element as a last child of a target element */
void   AddAsLastChild(
  WmqiContext *ctx,
  CciElement *targetElement,
  CciElement *newElement
);

/*Detach the target element */
void   Detach(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the parent of a target element */
CciElement  *Parent(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the previous sibling of a target element */
CciElement  *PreviousSibling(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the next sibling of a target element */
CciElement  *NextSibling(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the first child of a target element */
CciElement  *FirstChild(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the last child of a target element */
CciElement  *LastChild(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Set the type attribute of a target element */
void   SetElementType(
  WmqiContext *ctx,
  CciElement *targetElement,
  int type
);

/*Set the name attribute of a target element */
void   SetElementName(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *name
);

/*Set the value attribute of a target element */
void   SetElementValueValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  CciElementValue*value
);

/*Set the value of a target element */
void   SetElementBooleanValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int     value
);

/*Set the value of a target element */
void   SetElementIntegerValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int      value
);

/*Set the value of a target element */
void   SetElementRealValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  double     value);

/*Set the value of a target element */
void   SetElementCharacterValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  char       *value, int        length
);

/*Set the value of a target element */
void   SetElementDateValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int year, int month, int day
);

/*Set the value of a target element */
void   SetElementTimeValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int hour, int minute, float second
);

/*Set the value of a target element */
void   SetElementByteArrayValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  const struct CciByteArray*value
);

/*Set the value of a target element */
void   SetElementTimestampValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int year, int month, int day, int hour, int minute, float second
);

/*Set the value of a target element */
void   SetElementGmtTimestampValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int year, int month, int day, int hour, int minute, float second
);

/*Set the value of a target element */
void   SetElementGmtTimeValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  int hour, int minute, float second
  );

/*Set the value of a target element */
void   SetElementDecimalValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  char   *value
);

/*Set the value of a target element */
void   SetElementBitArrayValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  const struct CciBitArray*value
);

/*Return the type attribute of a target element */
int   ElementType(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the name attribute of a target element */
#newobject  ElementName
char *   ElementName(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
const CciElementValue  *ElementValueValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
int   ElementBooleanValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
int   ElementIntegerValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
double   ElementRealValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
#newobject  ElementCharacterValue
char *   ElementCharacterValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
struct CciDate  ElementDateValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
struct CciTime   ElementTimeValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
int   ElementByteArrayValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  const struct CciByteArray*value
);

/*Return the value of a target element */
struct CciTimestamp   ElementTimestampValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
struct CciTimestamp   ElementGmtTimestampValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
struct CciTime   ElementGmtTimeValue(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the value of a target element */
#newobject ElementDecimalValue
char *   ElementDecimalValue(
  WmqiContext *ctx,
  CciElement *targetElement

);

/*Return the value of a target element */
int   ElementBitArrayValue(
  WmqiContext *ctx,
  CciElement *targetElement,
  const struct CciBitArray*value
);

/*Copy part of an element tree to a target element */
void   CopyElementTree(
  WmqiContext *ctx,
  CciElement *sourceElement,
  CciElement *targetElement
);

/*Search the previous sibling chain for a matching element */
CciElement  *SearchPreviousSibling(
  WmqiContext *ctx,
  CciElement *targetElement,
  int mode,
  int type,
  char *name
);

/*Search the next sibling chain for a matching element */
CciElement  *SearchNextSibling(
  WmqiContext *ctx,
  CciElement *targetElement,
  int mode,
  int type,
  char *name
);

/*Search the sibling chain from the first child for a matching element */
CciElement  *SearchFirstChild(
  WmqiContext *ctx,
  CciElement *targetElement,
  int mode,
  int type,
  char *name
);

/*Search the sibling chain from the last child for a matching element */
CciElement  *SearchLastChild(
  WmqiContext *ctx,
  CciElement *targetElement,
  int mode,
  int type,
  char *name
);

/*Return the type attribute of the value of a target element */
int   ElementValueType(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Return the state attribute of the value of a target element */
int   ElementValueState(
  WmqiContext *ctx,
  CciElement *targetElement
);

/*Create an SQL statement */
CciSqlExpression  *SqlCreateStatement(
  WmqiContext *ctx,
  char   *dataSourceName,
  int transaction,
  char   *statement
);

/*Execute an SQL expression */
void   SqlExecute(
  WmqiContext *ctx,
  CciSqlExpression   *sqlExpression
);

/*Execute an SQL expression */
void   SqlSelect(
  WmqiContext *ctx,
  CciSqlExpression   *sqlExpression,
  CciMessage   *outputMessage
); 

/*Delete an SQL statement */
void   SqlDeleteStatement(
  WmqiContext *ctx,
  CciSqlExpression   *sqlExpression
);

/*Get the environment for a message */
CciMessage  *GetEnvironmentMessage(
  WmqiContext *ctx,
  CciMessage   *message
);


/* Propagate a message to a named terminal on the node instance */
int Propagate(
  WmqiContext *ctx,
  CciMessage*  message
);


/* Create a message object */
CciMessage * CreateMessage(
  WmqiContext *ctx
);

/* Delete a message object */
void DeleteMessage(
  WmqiContext *ctx,
  CciMessage* message
);	